@@ -3,17 +3,29 @@ from abc import ABC
 from asyncio import Task, create_task, to_thread
 from collections import defaultdict
 from dataclasses import dataclass
-from functools import partial
+from functools import partial, wraps
 from types import MethodType, UnionType
-from typing import Annotated, Any, Protocol, Union, cast, get_args
+from typing import (
+    Annotated,
+    Any,
+    Awaitable,
+    Callable,
+    Generic,
+    Protocol,
+    TypeVar,
+    Union,
+    cast,
+    get_args,
+)
+from typing import get_origin as ty_get_origin
 from weakref import ref
 
 from ididi import Graph, Resolver
 from ididi.interfaces import GraphIgnore
 
-from lihil.ds.event import Envelope, Event
-from lihil.interface import MISSING, Record, lhl_get_origin
-from lihil.utils.typing import all_subclasses
+from lihil.interface import MISSING, Maybe, P, R, Struct
+from lihil.signature import EndpointSignature, Param
+from lihil.utils.typing import all_subclasses, get_origin_pro
 
 UNION_META = (UnionType, Union)
 CTX_MARKER = "__anywise_context__"
@@ -23,11 +35,13 @@ FrozenContext = Any
 IGNORE_TYPES = (Context, FrozenContext)
 """
 TODO:
-
 MsgCtx[T] = Annotated[T, CTX_MARKER]
 """
 
 
+E = TypeVar("E")
+
+
 # ============= Strategy ==============
 async def default_send(message: Any, context: Any, handler: Any) -> Any:
     return await handler(message, context)
@@ -175,7 +189,7 @@ def gather_types(annotation: Any) -> set[Any]:
         # raise Exception?
         return types
 
-    origin = lhl_get_origin(annotation)
+    origin = ty_get_origin(annotation)
     if not origin:
         types.add(annotation)
         types |= all_subclasses(annotation)
@@ -457,12 +471,12 @@ def get_methodmetas(msg_base: type, cls: type) -> list[MethodMeta]:
 
 
 # TODO: separate EventRegistry and CommandRegistry
-class MessageRegistry:
+class MessageRegistry(Generic[E]):
     def __init__(
         self,
         *,
         command_base: Any = MISSING,
-        event_base: Any = MISSING,
+        event_base: Maybe[type[E]] = MISSING,
         graph: Any = MISSING,
     ):
         self._command_base = command_base
@@ -482,7 +496,7 @@ class MessageRegistry:
         return self._command_base
 
     @property
-    def event_base(self) -> Any:
+    def event_base(self) -> Maybe[type[E]]:
         return self._event_base
 
     def __repr__(self) -> str:
@@ -605,7 +619,7 @@ class MessageRegistry:
                 self.guard_mapping[target].append(meta)
 
 
-class EventBus(Record):
+class EventBus(Struct, Generic[E], frozen=True):
     resolver: Resolver
     lsnmgr: ListenerManager
     strategy: Any
@@ -614,7 +628,7 @@ class EventBus(Record):
 
     async def publish(
         self,
-        event: Event,
+        event: E,
         *,
         context: Any = None,
     ) -> None:
@@ -626,11 +640,11 @@ class EventBus(Record):
 
     def emit(
         self,
-        event: Event,
+        event: E,
         context: Any = None,
         callback: Any = None,
     ) -> None:
-        async def event_task(event: Event, context: Any):
+        async def event_task(event: E, context: Any):
             async with self.resolver.ascope() as asc:
                 listeners = await self.lsnmgr.resolve_listeners(
                     type(event), resolver=asc
@@ -655,12 +669,15 @@ class EventBus(Record):
             raise SinkUnsetError()
 
 
-class BusTerminal:
+PEventBus = Annotated[EventBus[Any], Param("plugin")]
+
+
+class BusTerminal(Generic[E]):
     "persist meta data of event bus and command bus"
 
     def __init__(
         self,
-        *registries: Any,
+        *registries: MessageRegistry[E],
         graph: Any = None,
         sink: Any = None,
         sender: Any = default_send,
@@ -679,7 +696,7 @@ class BusTerminal:
         self.include(*registries)
 
     def create_event_bus(self, resolver: Resolver):
-        return EventBus(
+        return EventBus[E](
             resolver, self._listener_manager, self._publisher, self._sink, self._tasks
         )
 
@@ -726,3 +743,26 @@ class BusTerminal:
 
         handler = await self._handler_manager.resolve_handler(type(msg), resolver)
         return await self._sender(msg, context, handler)
+
+
+class BusPlugin:
+    def __init__(self, busterm: BusTerminal[Any]):
+        self.busterm = busterm
+
+    async def decorate(
+        self, graph: Graph, func: Callable[P, Awaitable[R]], sig: EndpointSignature[Any]
+    ) -> Callable[P, Awaitable[R]]:
+        for name, param in sig.plugins.items():
+            param_type, _ = get_origin_pro(param.type_)
+            param_type = ty_get_origin(param_type) or param_type
+            if param_type is EventBus:
+                break
+        else:
+            return func
+
+        @wraps(func)
+        async def f(*args: P.args, **kwargs: P.kwargs) -> R:
+            kwargs[name] = self.busterm.create_event_bus(graph)
+            return await func(*args, **kwargs)
+
+        return f
