@@ -1,17 +1,44 @@
 from collections.abc import Mapping
+from dataclasses import is_dataclass
 from types import GenericAlias, UnionType
 from typing import (
     Annotated,
     Any,
+    ForwardRef,
     Literal,
     Sequence,
-    TypeAliasType,
     TypeVar,
     Union,
     cast,
     get_args,
 )
 from typing import get_origin as ty_get_origin
+from typing import overload
+
+from msgspec import Struct
+from pydantic import BaseModel
+from typing_extensions import TypeAliasType, TypeGuard, is_typeddict
+
+T = TypeVar("T")
+
+
+@overload
+def lenient_issubclass(t: Any, cls_or_clses: type[T]) -> TypeGuard[type[T]]: ...
+
+
+@overload
+def lenient_issubclass(t: Any, cls_or_clses: tuple[type, ...]) -> TypeGuard[type]: ...
+
+
+def lenient_issubclass(
+    t: Any, cls_or_clses: Union[type[T], tuple[type, ...]]
+) -> TypeGuard[type[T]]:
+    try:
+        return isinstance(t, type) and issubclass(t, cls_or_clses)
+    except TypeError:
+        if origin := ty_get_origin(t):
+            return lenient_issubclass(origin, cls_or_clses)
+        return False
 
 
 def union_types(subs: Sequence[type[Any]]) -> type | UnionType | GenericAlias | None:
@@ -23,7 +50,7 @@ def union_types(subs: Sequence[type[Any]]) -> type | UnionType | GenericAlias |
         return None
     elif len(subs) == 1:
         return next(iter(subs))
-    return Union[*subs]  # type: ignore
+    return Union[tuple(subs)]  # type: ignore
 
 
 def is_py_singleton(t: Any) -> Literal[None, True, False]:
@@ -32,13 +59,18 @@ def is_py_singleton(t: Any) -> Literal[None, True, False]:
 
 def is_union_type(
     t: type | UnionType | GenericAlias | TypeAliasType,
-):
+) -> TypeGuard[UnionType]:
     return ty_get_origin(t) in (Union, UnionType)
 
 
-def is_nontextual_sequence(type_: Any, strict: bool = False):
+def is_nontextual_sequence(
+    type_: Any, strict: bool = False
+) -> TypeGuard[list[Any] | tuple[Any] | bytearray]:
     type_origin = ty_get_origin(type_) or type_
 
+    if type_origin is Union:
+        return any(is_nontextual_sequence(arg, strict) for arg in get_args(type_))
+
     if not isinstance(type_origin, type):
         return False
 
@@ -59,13 +91,24 @@ def is_text_type(t: type | UnionType | GenericAlias) -> bool:
     return t in (str, bytes)
 
 
-def is_mapping_type(qtype: type | UnionType | GenericAlias) -> bool:
-    if is_union_type(qtype):
-        q_union_args = get_args(qtype)
-        return any(is_mapping_type(q) for q in q_union_args)
+def is_structured_type(
+    param_type: type[Any] | UnionType | GenericAlias, homogeneous_union: bool = False
+) -> bool:
+    if is_union_type(param_type):
+        q_union_args = get_args(param_type)
+        union_filter = all if homogeneous_union else any
+        return union_filter(is_structured_type(q) for q in q_union_args)
 
-    qorigin = ty_get_origin(qtype) or qtype
-    return qorigin is dict or isinstance(qorigin, type) and issubclass(qorigin, Mapping)
+    qorigin = ty_get_origin(param_type) or param_type
+
+    if not isinstance(qorigin, type):
+        return False
+
+    return (
+        issubclass(qorigin, (Mapping, Struct, BaseModel))
+        or is_typeddict(qorigin)
+        or is_dataclass(qorigin)
+    )
 
 
 def is_generic_type(type_: Any) -> bool:
@@ -75,32 +118,26 @@ def is_generic_type(type_: Any) -> bool:
 
 
 def replace_typevars(
-    tyvars: tuple[type | TypeVar, ...], nonvars: tuple[type, ...]
+    tyvars: Sequence[type | TypeVar], nonvars: Sequence[type]
 ) -> tuple[type, ...]:
     typevar_map: dict[TypeVar, None | type] = {
         item: None for item in tyvars if isinstance(item, TypeVar)
     }
 
-    if len(typevar_map) > len(nonvars):
-        raise ValueError(f"Expected {len(typevar_map)} types in t2, got {len(nonvars)}")
+    var_size, nonvar_size = len(typevar_map), len(nonvars)
+    if var_size > nonvar_size:
+        raise ValueError(f"Expected {var_size} types in t2, got {nonvar_size}")
 
     idx = 0
-    for k in typevar_map:
-        typevar_map[k] = nonvars[idx]
-        idx += 1
+    for var in tyvars:
+        if isinstance(var, TypeVar) and typevar_map.get(var) is None:
+            typevar_map[var] = nonvars[idx]
+            idx += 1
 
-    # Replace TypeVars in t1
-    result: list[type] = []
-
-    for item in tyvars:
-        if isinstance(item, TypeVar):
-            mapped = typevar_map[item]
-            assert mapped
-            result.append(mapped)
-        else:
-            result.append(item)
-
-    return tuple(result)
+    result = tuple(
+        typevar_map[var] if isinstance(var, TypeVar) else var for var in tyvars
+    )
+    return result
 
 
 def repair_type_generic_alias(
@@ -130,7 +167,7 @@ def recursive_get_args(type_: Any) -> tuple[Any, ...]:
     return type_args
 
 
-def deannotate[T](
+def deannotate(
     annt: Annotated[type[T], "Annotated"] | UnionType | GenericAlias,
 ) -> tuple[type[T], list[Any]] | tuple[type[T], None]:
     type_args = get_args(annt)
@@ -149,22 +186,13 @@ def deannotate[T](
                 flattened_metadata.extend(metas)
         else:
             flattened_metadata.append(item)
-    # if isinstance(ty_get_origin(atype), TypeAliasType):
-    #     new_type, new_metas = deannotate(atype.__value__)
-    #     new_type: type[T]
-    #     if new_metas:
-    #         merged: list[Any] = new_metas + flattened_metadata
-    #     else:
-    #         merged = flattened_metadata
-    #     return (new_type, merged)
     return (atype, flattened_metadata)
 
 
-# TODO: carry type_args, StriDict[int] -> tyargs = ()
-def get_origin_pro[T](
-    type_: type[T] | UnionType | GenericAlias | TypeAliasType | TypeVar,
+def get_origin_pro(
+    type_: type[T] | UnionType | GenericAlias | TypeAliasType | TypeVar | ForwardRef,
     metas: list[Any] | None = None,
-    type_args: tuple[type] | None = None,
+    type_args: tuple[type, ...] | None = None,
 ) -> tuple[type | UnionType | GenericAlias, list[Any] | None]:
     """
     type MyTypeAlias = Annotated[Query[int], CustomEncoder]
@@ -178,6 +206,9 @@ def get_origin_pro[T](
     if isinstance(type_, TypeAliasType):
         return get_origin_pro(type_.__value__, metas, type_args)
 
+    if isinstance(type_, ForwardRef):
+        raise TypeError(f"{type_} is not supported before python 3.12")
+
     if (current_origin := ty_get_origin(type_)) is None:
         return (cast(type, type_), cast(None, metas))
 
@@ -189,7 +220,8 @@ def get_origin_pro[T](
         if local_metas and metas:
             local_metas += metas
         return get_origin_pro(annt_type, local_metas, type_args)
-    elif isinstance(current_origin, TypeAliasType):
+
+    if isinstance(current_origin, TypeAliasType):
         dealiased = cast(TypeAliasType, type_).__value__
         dtype, demetas = get_origin_pro(dealiased, metas, type_args)
 
@@ -204,14 +236,13 @@ def get_origin_pro[T](
                     for idx, meta in enumerate(demetas):
                         if isinstance(meta, TypeVar):
                             demetas[idx] = nontyvar.pop(0)
-
                 return get_origin_pro(dtype, demetas, type_args)
             else:
                 dtype = repair_type_generic_alias(type_, type_args)
                 return dtype, None
         else:
             return get_origin_pro(dealiased, metas, type_args)
-    elif current_origin is UnionType:
+    if current_origin is UnionType:
         union_args = get_args(type_)
         utypes: list[type | UnionType | GenericAlias] = []
         new_metas: list[Any] = []
@@ -221,20 +252,18 @@ def get_origin_pro[T](
             if umeta:
                 new_metas.extend(umeta)
         if not new_metas:
-            return get_origin_pro(Union[*utypes], metas, type_args)  # type: ignore
+            return get_origin_pro(Union[tuple(utypes)], metas, type_args)  # type: ignore
 
         if metas is None:
             metas = new_metas
         else:
             metas.extend(new_metas)
-        return get_origin_pro(Union[*utypes], metas, type_args)  # type: ignore
-    else:
-        return (cast(type, type_), cast(None, metas))
+        return get_origin_pro(Union[tuple(utypes)], metas, type_args)  # type: ignore
+
+    return (cast(type, type_), cast(None, metas))
 
 
-def all_subclasses[T](
-    cls: type[T], ignore: set[type[Any]] | None = None
-) -> set[type[T]]:
+def all_subclasses(cls: type[T], ignore: set[type[Any]] | None = None) -> set[type[T]]:
     """
     Get all subclasses of a class recursively, avoiding repetition.
 
