@@ -1,42 +1,31 @@
 import uuid
-from types import GenericAlias, UnionType
-from typing import Annotated, Any, Literal
+from typing import Annotated
 
 import pytest
 from ididi import AsyncScope, Graph, Ignore, use
 from starlette.requests import Request
 
 from lihil import (
-    Cookie,
     Empty,
     Form,
-    Header,
     Json,
-    Meta,
+    Param,
     Payload,
-    Query,
     Request,
-    Resolver,
-    Resp,
     Route,
     Stream,
     Text,
     UploadFile,
-    Use,
     field,
     status,
 )
-from lihil.auth.jwt import JWTAuth, JWTPayload, jwt_decoder_factory
-from lihil.auth.oauth import OAuth2PasswordFlow, OAuthLoginForm
-from lihil.config import AppConfig, SecurityConfig
-from lihil.errors import (
-    InvalidParamTypeError,
-    MissingDependencyError,
-    NotSupportedError,
-    StatusConflictError,
-)
-from lihil.plugins.registry import PluginBase, PluginParam
-from lihil.plugins.testclient import LocalClient
+from lihil.config import DEFAULT_CONFIG, lhl_set_config
+from lihil.errors import InvalidParamError, StatusConflictError
+from lihil.interface import Base
+from lihil.local_client import LocalClient
+from lihil.plugins.auth.jwt import JWTAuthParam, JWTAuthPlugin, JWTConfig
+from lihil.plugins.auth.oauth import OAuth2PasswordFlow, OAuthLoginForm
+from lihil.signature.parser import EndpointParser
 from lihil.utils.threading import async_wrapper
 from lihil.utils.typing import is_nontextual_sequence
 
@@ -57,7 +46,12 @@ def rusers() -> Route:
 
 @pytest.fixture
 def testroute() -> Route:
-    return Route("test")
+    app_config = JWTConfig(JWT_SECRET="mysecret", JWT_ALGORITHMS=["HS256"])
+    lhl_set_config(app_config)
+    route = Route("test")
+    route.endpoint_parser = EndpointParser(route.graph, route.path)
+    yield route
+    lhl_set_config(DEFAULT_CONFIG)
 
 
 @pytest.fixture
@@ -69,19 +63,18 @@ def add_q(q: str, user_id: str) -> Ignore[str]:
     return q
 
 
-async def create_user(
-    user: User,
-    req: Request,
-    user_id: str,
-    func_dep: Annotated[str, use(add_q)],
-) -> Resp[Json[User], status.CREATED]:
-    return User(id=user.id, name=user.name, email=user.email)
+async def test_return_status(rusers: Route):
+    async def create_user(
+        user: User,
+        req: Request,
+        user_id: str,
+        func_dep: Annotated[str, use(add_q)],
+    ) -> Annotated[Json[User], status.CREATED]:
+        return User(id=user.id, name=user.name, email=user.email)
 
-
-def test_return_status(rusers: Route):
     rusers.post(create_user)
+    await rusers.setup()
     ep = rusers.get_endpoint(create_user)
-    ep.setup()
     assert "q" in ep.sig.query_params
     assert "func_dep" in ep.sig.dependencies
     assert "user_id" in ep.sig.path_params
@@ -90,26 +83,26 @@ def test_return_status(rusers: Route):
     assert ep_ret.type_ is User
 
 
-def test_status_conflict(rusers: Route):
+async def test_status_conflict(rusers: Route):
 
     async def get_user(
         user_id: str,
-    ) -> Annotated[Resp[str, status.NO_CONTENT], "hello"]:
+    ) -> Annotated[Annotated[str, status.NO_CONTENT], "hello"]:
         return "hello"
 
     rusers.get(get_user)
     with pytest.raises(StatusConflictError):
-        ep = rusers.get_endpoint(get_user)
-        ep.setup()
+        rusers.get_endpoint(get_user)
+        await rusers.setup()
 
 
-def test_annotated_generic(rusers: Route):
+async def test_annotated_generic(rusers: Route):
 
     async def update_user(user_id: str) -> Annotated[dict[str, str], "aloha"]: ...
 
     rusers.put(update_user)
     ep = rusers.get_endpoint(update_user)
-    ep.setup()
+    await rusers.setup()
     repr(ep)
     assert ep.sig.return_params[200].type_ == dict[str, str]
 
@@ -207,13 +200,13 @@ async def test_scoped_endpoint(rusers: Route, lc: LocalClient):
     await lc.call_endpoint(ep)
 
 
-@pytest.mark.debug
 async def test_ep_drop_body(rusers: Route, lc: LocalClient):
 
-    async def get() -> Resp[Empty, 400]:
+    async def get() -> Annotated[Empty, status.BAD_REQUEST]:
         return "asdf"
 
     rusers.get(get)
+    await rusers.setup()
     ep = rusers.get_endpoint("GET")
 
     res = await lc.call_endpoint(ep)
@@ -228,7 +221,9 @@ async def test_ep_requiring_form(rusers: Route, lc: LocalClient):
         username: str
         email: str
 
-    async def get(req: Request, fm: Form[UserInfo]) -> Resp[str, status.OK]:
+    async def get(
+        req: Request, fm: Annotated[UserInfo, Form()]
+    ) -> Annotated[str, status.OK]:
         return fm
 
     rusers.get(get)
@@ -261,13 +256,15 @@ async def test_ep_requiring_form(rusers: Route, lc: LocalClient):
     assert res
 
 
-async def test_ep_requiring_missing_param(rusers: Route, lc: LocalClient):
+async def test_ep_requiring_missing_Param(rusers: Route, lc: LocalClient):
 
     class UserInfo(Payload):
         username: str
         email: str
 
-    async def get(req: Request, fm: Form[UserInfo]) -> Resp[str, 200]:
+    async def get(
+        req: Request, fm: Annotated[UserInfo, Form()]
+    ) -> Annotated[str, status.OK]:
         return fm
 
     rusers.get(get)
@@ -300,11 +297,9 @@ async def test_ep_requiring_missing_param(rusers: Route, lc: LocalClient):
 
 async def test_ep_requiring_upload_file(rusers: Route, lc: LocalClient):
 
-    class UserInfo(Payload):
-        username: str
-        email: str
-
-    async def get(req: Request, myfile: UploadFile) -> Resp[str, 200]:
+    async def get(
+        req: Request, myfile: Annotated[UploadFile, Form()]
+    ) -> Annotated[str, status.OK]:
         assert isinstance(myfile, UploadFile)
         return None
 
@@ -330,8 +325,42 @@ async def test_ep_requiring_upload_file(rusers: Route, lc: LocalClient):
     assert result.status_code == 200
 
 
+async def test_ep_requiring_upload_file_exceed_max_files(
+    rusers: Route, lc: LocalClient
+):
+
+    async def get(
+        req: Request, myfile: Annotated[UploadFile, Form(max_files=0)]
+    ) -> Annotated[str, status.OK]:
+        assert isinstance(myfile, UploadFile)
+        return None
+
+    boundary = f"----WebKitFormBoundary{uuid.uuid4().hex}"
+
+    file_content = b"Hello, this is test content!"  # Example file content
+    filename = "test_file.txt"
+
+    multipart_body = (
+        f"--{boundary}\r\n"
+        f'Content-Disposition: form-data; name="myfile"; filename="{filename}"\r\n'
+        f"Content-Type: text/plain\r\n\r\n"
+        + file_content.decode()  # File content as string
+        + f"\r\n--{boundary}--\r\n"
+    ).encode("utf-8")
+
+    headers = {"Content-Type": f"multipart/form-data; boundary={boundary}"}
+
+    rusers.get(get)
+    ep = rusers.get_endpoint("GET")
+
+    result = await lc.call_endpoint(ep, body=multipart_body, headers=headers)
+    assert result.status_code == 422
+    data = await result.json()
+    assert data["detail"][0]["type"] == "InvalidFormError"
+
+
 async def test_ep_requiring_upload_file_fail(rusers: Route, lc: LocalClient):
-    async def get(req: Request, myfile: UploadFile) -> Resp[str, 200]:
+    async def get(req: Request, myfile: UploadFile) -> Annotated[str, status.OK]:
         return None
 
     rusers.get(get)
@@ -342,7 +371,9 @@ async def test_ep_requiring_upload_file_fail(rusers: Route, lc: LocalClient):
 
 
 async def test_ep_requiring_file_bytse(rusers: Route, lc: LocalClient):
-    async def get(by_form: Form[bytes]) -> Resp[Text, 200]:
+    async def get(
+        by_form: Annotated[list[int], Form()],
+    ) -> Annotated[Text, status.OK]:
         assert isinstance(by_form, bytes)
         return "ok"
 
@@ -367,23 +398,24 @@ async def test_ep_requiring_file_bytse(rusers: Route, lc: LocalClient):
     # Content-Type header
     content_type = f"multipart/form-data; boundary={boundary}"
 
-    res = await lc.call_endpoint(
-        ep,
-        body=multipart_data,
-        headers={f"content-type": content_type},
-    )
-    assert await res.text() == "ok"
-    assert res.status_code == 200
+    with pytest.raises(InvalidParamError):
+        await lc.call_endpoint(
+            ep,
+            body=multipart_data,
+            headers={f"content-type": content_type},
+        )
 
 
 async def test_ep_requiring_form_invalid_type(rusers: Route, lc: LocalClient):
-    async def get(by_form: Form[list[int]]) -> Resp[Text, 200]:
+    async def get(
+        by_form: Annotated[list[int], Form()],
+    ) -> Annotated[Text, status.OK]:
         assert isinstance(by_form, bytes)
         return "ok"
 
     rusers.get(get)
-    with pytest.raises(NotSupportedError):
-        rusers.get_endpoint("GET").setup()
+    with pytest.raises(InvalidParamError):
+        await rusers.setup()
 
 
 async def test_ep_requiring_form_sequence_type(rusers: Route, lc: LocalClient):
@@ -391,7 +423,9 @@ async def test_ep_requiring_form_sequence_type(rusers: Route, lc: LocalClient):
         name: str
         phones: list[str]
 
-    async def get(by_form: Form[UserInfo]) -> Resp[Text, status.OK]:
+    async def get(
+        by_form: Annotated[UserInfo, Param()],
+    ) -> Annotated[Text, status.OK]:
         assert isinstance(by_form, UserInfo)
         return "ok"
 
@@ -403,26 +437,28 @@ async def test_ep_mark_override_others(rusers: Route, lc: LocalClient):
         name: str
         phones: list[str]
 
-    async def get(user_id: Query[UserInfo]) -> Resp[Text, status.OK]:
+    async def get(
+        user_id: Annotated[UserInfo, Param("query")],
+    ) -> Annotated[Text, status.OK]:
         return "ok"
 
     rusers.get(get)
 
     ep = rusers.get_endpoint("GET")
-    ep.setup()
+    await rusers.setup()
     assert ep.sig.query_params
     assert not ep.sig.path_params
 
 
 async def test_ep_with_random_annoated_query(rusers: Route, lc: LocalClient):
 
-    async def get(aloha: Annotated[int, "aloha"]) -> Resp[Text, status.OK]:
+    async def get(aloha: Annotated[int, "aloha"]) -> Annotated[Text, status.OK]:
         return "ok"
 
     rusers.get(get)
 
     ep = rusers.get_endpoint("GET")
-    ep.setup()
+    await rusers.setup()
     assert ep.sig.query_params
     assert "aloha" in ep.sig.query_params
     assert ep.sig.query_params["aloha"].type_ is int
@@ -430,13 +466,13 @@ async def test_ep_with_random_annoated_query(rusers: Route, lc: LocalClient):
 
 async def test_ep_with_random_annoated_path1(rusers: Route, lc: LocalClient):
 
-    async def get(user_id: Annotated[int, "aloha"]) -> Resp[Text, status.OK]:
+    async def get(user_id: Annotated[int, "aloha"]) -> Annotated[Text, status.OK]:
         return "ok"
 
     rusers.get(get)
 
     ep = rusers.get_endpoint("GET")
-    ep.setup()
+    await rusers.setup()
     assert ep.sig.path_params
     assert "user_id" in ep.sig.path_params
     assert ep.sig.path_params["user_id"].type_ is int
@@ -447,13 +483,13 @@ async def test_ep_with_random_annoated_path2(rusers: Route, lc: LocalClient):
         name: str
         phones: list[str]
 
-    async def get(user: Annotated[UserInfo, "aloha"]) -> Resp[Text, status.OK]:
+    async def get(user: Annotated[UserInfo, "aloha"]) -> Annotated[Text, status.OK]:
         return "ok"
 
     rusers.get(get)
 
     ep = rusers.get_endpoint("GET")
-    ep.setup()
+    await rusers.setup()
     assert ep.sig.body_param
     assert ep.sig.body_param[1].type_ is UserInfo
 
@@ -470,7 +506,7 @@ async def test_ep_require_resolver(rusers: Route, lc: LocalClient):
 
     async def get(
         user_id: str, engine: Engine, resolver: Graph
-    ) -> Resp[Text, status.OK]:
+    ) -> Annotated[Text, status.OK]:
         await resolver.aresolve(call_back)
         return "ok"
 
@@ -487,8 +523,8 @@ async def test_config_nonscoped_ep_to_be_scoped(rusers: Route, lc: LocalClient):
     class Engine: ...
 
     async def get(
-        user_id: str, engine: Use[Engine], resolver: AsyncScope
-    ) -> Resp[Text, status.OK]:
+        user_id: str, engine: Annotated[Engine, use(Engine)], resolver: AsyncScope
+    ) -> Annotated[Text, status.OK]:
         with pytest.raises(AssertionError):
             assert isinstance(resolver, AsyncScope)
         return "ok"
@@ -502,8 +538,8 @@ async def test_config_nonscoped_ep_to_be_scoped(rusers: Route, lc: LocalClient):
     assert text == "ok"
 
     async def post(
-        user_id: str, engine: Use[Engine], resolver: AsyncScope
-    ) -> Resp[Text, status.OK]:
+        user_id: str, engine: Annotated[Engine, use(Engine)], resolver: AsyncScope
+    ) -> Annotated[Text, status.OK]:
         assert isinstance(resolver, AsyncScope)
         return "ok"
 
@@ -516,7 +552,7 @@ async def test_config_nonscoped_ep_to_be_scoped(rusers: Route, lc: LocalClient):
     assert text == "ok"
 
 
-type GET_RESP = Resp[Text, status.OK]
+GET_RESP = Annotated[Text, status.OK]
 
 
 async def test_endpoint_with_resp_alias(rusers: Route, lc: LocalClient):
@@ -533,106 +569,100 @@ async def test_endpoint_with_resp_alias(rusers: Route, lc: LocalClient):
     assert text == "ok"
 
 
-class UserProfile(JWTPayload):
-    __jwt_claims__ = {"expires_in": 3600}
+class UserProfile(Base):
 
     user_id: str = field(name="sub")
     user_name: str
 
 
-async def test_endpoint_returns_jwt_payload(testroute: Route, lc: LocalClient):
+# async def test_endpoint_returns_jwt_payload(testroute: Route, lc: LocalClient):
 
-    async def get_token(form: OAuthLoginForm) -> JWTAuth[UserProfile]:
-        return UserProfile(user_id="1", user_name=form.username)
+#     async def get_token(data: OAuthLoginForm) -> JWTAuth[UserProfile]:
+#         return UserProfile(user_id="1", user_name=data.username)
 
-    testroute.post(get_token)
+#     testroute.post(get_token)
 
-    ep = testroute.get_endpoint(get_token)
+#     ep = testroute.get_endpoint(get_token)
 
-    testroute.app_config = AppConfig(
-        security=SecurityConfig(jwt_secret="mysecret", jwt_algorithms=["HS256"])
-    )
-    ep.setup()
+#     app_config = JWTConfig(JWT_SECRET="mysecret", JWT_ALGORITHMS=["HS256"])
+#     lhl_set_config(app_config)
+#     await testroute.setup()
 
-    res = await lc.submit_form(
-        ep, form_data={"username": "user", "password": "pasword"}
-    )
+#     res = await lc.submit_form(
+#         ep, form_data={"username": "user", "password": "pasword"}
+#     )
 
-    token = await res.json()
+#     token = await res.json()
 
-    decoder = jwt_decoder_factory(
-        secret="mysecret", algorithms=["HS256"], payload_type=UserProfile
-    )
+#     decoder = jwt_decoder_factory(payload_type=UserProfile)
+#     token_type, token = token["token_type"], token["access_token"]
+#     content = f"{token_type.capitalize()} {token}"
 
-    content = f"{token["token_type"].capitalize()} {token["access_token"]}"
-
-    payload = decoder(content)
-    assert isinstance(payload, UserProfile)
+#     payload = decoder(content)
+#     assert isinstance(payload, UserProfile)
 
 
 async def test_oauth2_not_plugin():
 
-    async def get_user(token: Header[str, Literal["Authorization"]]): ...
+    async def get_user(
+        token: Annotated[str, Param("header", alias="Authorization")],
+    ): ...
 
     route = Route("me")
     route.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_user)
 
     ep = route.get_endpoint("GET")
-    ep.setup()
+    await route.setup()
 
-    pg = ep.sig.plugins
-    assert not pg
+    assert ep.sig.header_params
 
 
-async def test_endpoint_with_jwt_decode_fail(testroute: Route, lc: LocalClient):
-    async def get_me(token: JWTAuth[UserProfile]):
+@pytest.fixture
+def jwt_auth_plugin():
+    return JWTAuthPlugin(jwt_secret="mysecret", jwt_algorithms="HS256")
+
+
+async def test_endpoint_with_jwt_decode_fail(
+    testroute: Route, lc: LocalClient, jwt_auth_plugin: JWTAuthPlugin
+):
+    async def get_me(
+        token: Annotated[
+            UserProfile,
+            Param("header", alias="Authorization", extra_meta=dict(skip_unpack=True)),
+        ],
+    ):
         assert isinstance(token, UserProfile)
 
-    testroute.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_me)
-
-    testroute.app_config = AppConfig(
-        security=SecurityConfig(jwt_secret="mysecret", jwt_algorithms=["HS256"])
-    )
+    testroute.get(
+        auth_scheme=OAuth2PasswordFlow(token_url="token"),
+        plugins=[jwt_auth_plugin.decode_plugin],
+    )(get_me)
 
     ep = testroute.get_endpoint(get_me)
-    ep.setup()
+    await testroute.setup()
 
     res = await lc(ep, headers={"Authorization": "adsfjaklsdjfklajsdfkjaklsdfj"})
     assert res.status_code == 401
 
 
-async def test_endpoint_with_jwt_fail_without_security_config(
-    testroute: Route, lc: LocalClient
+async def test_endpoint_login_and_validate(
+    testroute: Route, lc: LocalClient, jwt_auth_plugin: JWTAuthPlugin
 ):
-    async def get_me(token: JWTAuth[UserProfile]):
-        assert isinstance(token, UserProfile)
-
-    testroute.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_me)
-
-    ep = testroute.get_endpoint(get_me)
-
-    with pytest.raises(MissingDependencyError):
-        ep.setup()
-
-
-async def test_endpoint_login_and_validate(testroute: Route, lc: LocalClient):
-    from lihil.config import lhl_set_config
-
-    async def get_me(token: JWTAuth[UserProfile]) -> Resp[Text, status.OK]:
+    @testroute.get(
+        auth_scheme=OAuth2PasswordFlow(token_url="token"),
+        plugins=[jwt_auth_plugin.decode_plugin],
+    )
+    async def get_me(
+        token: Annotated[UserProfile, JWTAuthParam],
+    ) -> Annotated[Text, status.OK]:
         assert token.user_id == "1" and token.user_name == "2"
         return "ok"
 
-    async def login_get_token(login_form: OAuthLoginForm) -> JWTAuth[UserProfile]:
+    @testroute.post(plugins=[jwt_auth_plugin.encode_plugin(expires_in_s=3600)])
+    async def login_get_token(login_form: OAuthLoginForm) -> UserProfile:
         return UserProfile(user_id="1", user_name="2")
 
-    testroute.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_me)
-    testroute.post(login_get_token)
-    lhl_set_config(
-        app_config=AppConfig(
-            security=SecurityConfig(jwt_secret="mysecret", jwt_algorithms=["HS256"])
-        )
-    )
-    testroute.setup()
+    await testroute.setup()
 
     login_ep = testroute.get_endpoint(login_get_token)
 
@@ -655,59 +685,6 @@ async def test_endpoint_login_and_validate(testroute: Route, lc: LocalClient):
     assert await res.text() == "ok"
 
 
-@pytest.mark.skip("not implemented")
-async def test_endpoint_login_and_validate_with_str_resp(
-    testroute: Route, lc: LocalClient
-):
-    async def get_me(token: JWTAuth[str]) -> Resp[Text, status.OK]:
-        assert token == "user_id"
-        return "ok"
-
-    async def login_get_token(login_form: OAuthLoginForm) -> JWTAuth[str]:
-        return "user_id"
-
-    testroute.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_me)
-    testroute.post(login_get_token)
-    set_config(
-        AppConfig(
-            security=SecurityConfig(jwt_secret="mysecret", jwt_algorithms=["HS256"])
-        )
-    )
-    testroute.setup()
-
-    login_ep = testroute.get_endpoint(login_get_token)
-
-    res = await lc.submit_form(
-        login_ep, form_data={"username": "user", "password": "test"}
-    )
-
-    token_data = await res.json()
-
-    token_type, token = token_data["token_type"], token_data["token"]
-    token_type: str
-
-    lc.update_headers({"Authorization": f"{token_type.capitalize()} {token}"})
-
-    meep = testroute.get_endpoint(get_me)
-
-    res = await lc(meep)
-
-    assert res.status_code == 200
-    assert await res.text() == "ok"
-
-
-async def test_ep_with_plugin_type(testroute: Route, lc: LocalClient):
-
-    class MyPlugin(PluginBase): ...
-
-    async def myep(param: Annotated[str, MyPlugin]): ...
-
-    testroute.get(myep)
-
-    with pytest.raises(NotSupportedError):
-        testroute.setup()
-
-
 async def test_ep_is_scoped(testroute: Route):
     class Engine: ...
 
@@ -718,64 +695,11 @@ async def test_ep_is_scoped(testroute: Route):
 
     testroute.get(func)
     ep = testroute.get_endpoint(func)
-    ep.setup()
+    await testroute.setup()
 
     assert ep.scoped
 
 
-async def test_ep_with_plugin(testroute: Route):
-    class MyParamProcessor(PluginBase):
-        def __init__(self, name: str):
-            self.name = name
-
-        async def process(
-            self, params: dict[str, Any], request: Request, resolver: Resolver
-        ) -> None:
-            params[self.name] = "processed"
-
-    called: bool = False
-
-    def func(p: Annotated[str, MyParamProcessor("p")]):
-        nonlocal called
-        called = True
-        assert p == "processed"
-
-    testroute.get(func)
-
-    ep = testroute.get_endpoint(func)
-    ep.setup()
-
-    assert ep.sig.plugins
-
-    await LocalClient()(ep)
-    assert called
-
-
-async def test_plugin_without_processor(testroute: Route):
-
-    class MyPlugin(PluginBase):
-        def parse(
-            self,
-            name: str,
-            type_: type | UnionType | GenericAlias,
-            annotation: Any,
-            default: Any,
-        ) -> PluginParam:
-            return PluginParam(
-                type_=type_,
-                annotation=annotation,
-                name=name,
-                default=default,
-                plugin=self,
-            )
-
-    async def f(p: Annotated[str, MyPlugin()]): ...
-
-    lc = LocalClient()
-    with pytest.raises(InvalidParamTypeError):
-        await lc(lc.make_endpoint(f))
-
-
 async def test_endpoint_with_list_query():
     called = False
 
@@ -786,7 +710,7 @@ async def test_endpoint_with_list_query():
 
     lc = LocalClient()
     res = await lc.request(
-        lc.make_endpoint(get_cart),
+        await lc.make_endpoint(get_cart),
         method="GET",
         path="/",
         query_string="names=5&names=6",
@@ -807,7 +731,7 @@ async def test_endpoint_with_tuple_query():
 
     lc = LocalClient()
     res = await lc.request(
-        lc.make_endpoint(get_cart),
+        await lc.make_endpoint(get_cart),
         method="GET",
         path="/",
         query_string=b"names=5&names=6",
@@ -826,14 +750,14 @@ async def test_ep_with_constraints():
     called: bool = False
 
     async def get_user(
-        n: Annotated[int, Meta(gt=0)], user_id: Annotated[str, Meta(min_length=5)]
+        n: Annotated[int, Param(gt=0)], user_id: Annotated[str, Param(min_length=5)]
     ):
         nonlocal called
         called = True
 
     lc = LocalClient()
 
-    ep = lc.make_endpoint(get_user, path="/{user_id}")
+    ep = await lc.make_endpoint(get_user, path="/{user_id}")
     resp = await lc(ep, path_params={"user_id": "user"}, query_params={"n": -1})
     res = await resp.json()
     assert not called
@@ -844,18 +768,18 @@ async def test_ep_with_cookie():
 
     async def get_user(
         refresh_token: Annotated[
-            Cookie[str, Literal["refresh-token"]], Meta(min_length=1)
+            str, Param("cookie", alias="x-refresh-token", min_length=1)
         ],
-        user_id: Annotated[str, Meta(min_length=5)],
+        user_id: Annotated[str, Param(min_length=5)],
     ):
         nonlocal called
         assert len(user_id) >= 5
         called = True
         return True
 
-    lc = LocalClient(headers={"cookie": "refresh-token=asdf"})
-
-    ep = lc.make_endpoint(get_user, path="/{user_id}")
+    lc = LocalClient(headers={"cookie": "x-refresh-token=asdf"})
+    ep = await lc.make_endpoint(get_user, path="/{user_id}")
+    assert ep.sig.header_params["refresh_token"].cookie_name == "x-refresh-token"
     resp = await lc(ep, path_params={"user_id": "user123"})
     res = await resp.json()
     assert res
@@ -866,8 +790,8 @@ async def test_ep_with_cookie2():
     called: bool = False
 
     async def get_user(
-        refresh_token: Annotated[Cookie[str], Meta(min_length=1)],
-        user_id: Annotated[str, Meta(min_length=5)],
+        refresh_token: Annotated[str, Param("cookie", min_length=1)],
+        user_id: Annotated[str, Param(min_length=5)],
     ):
         nonlocal called
         called = True
@@ -875,8 +799,52 @@ async def test_ep_with_cookie2():
 
     lc = LocalClient(headers={"cookie": "refresh-token=asdf"})
 
-    ep = lc.make_endpoint(get_user, path="/{user_id}")
+    ep = await lc.make_endpoint(get_user, path="/{user_id}")
     resp = await lc(ep, path_params={"user_id": "user123"})
     res = await resp.json()
     assert res
     assert called
+
+
+async def tests_calling_ep_query_without_default():
+    lc = LocalClient()
+
+    async def get_user(user_id: int): ...
+
+    resp = await lc(await lc.make_endpoint(get_user))
+    assert resp.status_code == 422
+
+
+async def test_ep_with_multiple_value_header():
+    lc = LocalClient()
+
+    async def read_items(x_token: Annotated[list[str] | None, Param("header")] = None):
+        return {"X-Token values": x_token}
+
+    ep = await lc.make_endpoint(read_items)
+    resp = await lc.request(
+        ep,
+        method="GET",
+        path="",
+        multi_headers=[("x-token", "value1"), ("x-token", "value2")],
+    )
+    assert resp.status_code == 200
+
+
+from msgspec import convert
+
+
+async def test_convert(): ...
+
+
+async def test_ep_requiring_upload_file_with_decoder(
+    rusers: Route, lc: LocalClient
+):
+
+    async def get(
+        req: Request, myfile: Annotated[UploadFile, Form(max_files=0, decoder=lambda x: x)]
+    ) -> Annotated[str, status.OK]:
+        assert isinstance(myfile, UploadFile)
+        return ""
+
+    await lc.make_endpoint(get)
