@@ -2,27 +2,14 @@ from typing import Annotated, Literal
 
 import pytest
 
-from lihil import BusTerminal, Graph, Text, status
-
-# from lihil.constant.resp import METHOD_NOT_ALLOWED_RESP
-from lihil.signature.params import ParamParser
+from lihil import Graph, Param, Text, status
+from lihil.ds.event import Event
 
 # from lihil.errors import InvalidParamTypeError
-from lihil.interface import ASGIApp, Empty, Header, IReceive, IScope, ISend, Resp
-from lihil.plugins.bus import Event
-from lihil.plugins.testclient import LocalClient
+from lihil.interface import ASGIApp, Empty, IReceive, IScope, ISend
+from lihil.local_client import LocalClient
 from lihil.routing import Route
-
-
-async def test_route_flyweight_pattern():
-    # Test flyweight pattern
-    route1 = Route("/test")
-    route2 = Route("/test")
-    assert route1 is route2
-
-    # Different paths create different instances
-    route3 = Route("/other")
-    assert route1 is not route3
+from lihil.signature import EndpointParser
 
 
 async def test_route_truediv_operator():
@@ -58,14 +45,13 @@ async def test_route_match():
 
     # Valid match
     scope = {"path": "/users/123"}
-    matched_scope = route.match(scope)
+    assert route.match(scope)
 
-    assert matched_scope is not None
-    assert matched_scope["path_params"] == {"user_id": "123"}
+    assert scope["path_params"] == {"user_id": "123"}
 
     # No match
     scope = {"path": "/posts/123"}
-    assert route.match(scope) is None
+    assert not route.match(scope)
 
 
 async def test_route_call_with_valid_method():
@@ -294,7 +280,7 @@ async def test_route_build_stack():
     assert not route.call_stacks
 
     # Build the stack
-    route.setup()
+    await route.setup()
 
     # Now call_stacks should have the GET method
     assert "GET" in route.call_stacks
@@ -330,31 +316,31 @@ async def test_route_factory():
     assert callable(node_factory)
 
 
-async def test_route_listen():
-    route = Route("/test")
+# async def test_route_listen():
+#     route = Route("/test")
 
-    # Create a simple listener
-    def test_listener(event: Event):
-        pass
+#     # Create a simple listener
+#     def test_listener(event: Event):
+#         pass
 
-    # Register the listener
-    route.listen(test_listener)
+#     # Register the listener
+#     route.listen(test_listener)
 
-    assert route.has_listener(test_listener)
+#     assert route.has_listener(test_listener)
 
-    # Verify the listener was registered
+# Verify the listener was registered
 
 
-async def test_route_with_listeners_param():
-    # Create a simple listener
-    def test_listener(event: Event):
-        pass
+# async def test_route_with_listeners_Param():
+#     # Create a simple listener
+#     def test_listener(event: Event):
+#         pass
 
-    # Create route with listeners
-    route = Route("/test", listeners=[test_listener])
+#     # Create route with listeners
+#     route = Route("/test", listeners=[test_listener])
 
-    # Verify the listener was registered
-    assert route.has_listener(test_listener)
+#     # Verify the listener was registered
+#     assert route.has_listener(test_listener)
 
 
 async def test_route_decorator_style():
@@ -553,31 +539,13 @@ async def test_route_add_middleware_sequence():
     assert result["mw2"] is True
 
 
-async def test_route_has_listener():
-    route = Route("/test")
-
-    # Create listeners
-    def listener1(event: Event):
-        pass
-
-    def listener2(event: Event):
-        pass
-
-    # Register only one listener
-    route.listen(listener1)
-
-    # Test has_listener
-    assert route.has_listener(listener1) is True
-    assert route.has_listener(listener2) is False
-
-
 async def test_route_on_lifespan():
     route = Route("aloha")
 
     async def get(): ...
 
     route.get(get)
-    route.setup()
+    await route.setup()
     assert route.call_stacks["GET"]
 
 
@@ -692,8 +660,7 @@ async def test_route_with_literal_resp():
     async def post_empty() -> Literal[None]: ...
 
     route.post(post_empty)
-    # with pytest.raises(InvalidParamTypeError):
-    route.setup()
+    await route.setup()
 
     route.endpoints["POST"].sig.return_params[200]
 
@@ -701,7 +668,7 @@ async def test_route_with_literal_resp():
 async def test_route_with_nested_empty_response():
     route = Route("empty")
 
-    async def post_empty() -> Resp[Empty, status.NO_CONTENT]: ...
+    async def post_empty() -> Annotated[Empty, status.NO_CONTENT]: ...
 
     route.post(post_empty)
 
@@ -715,9 +682,87 @@ async def test_route_with_nested_empty_response():
 
 
 async def test_parse_header_with_key():
-    parser = ParamParser(Graph())
+    parser = EndpointParser(Graph(), "test")
 
-    res = parser.parse_param("token", Header[str, Literal["Authorization"]])
-    param = res[0]
+    res = parser.parse_param(
+        "token", Annotated[str, Param("header", alias="Authorization")]
+    )
+    p = res[0]
 
-    assert param.alias == "authorization"
+    assert p.alias == "Authorization"
+
+
+def test_routing_merge_sub_route():
+    root = Route("api/v0")
+    user = Route("users")
+    item = user.sub("items")
+    item.sub("{item_id}")
+    root.include_subroutes(user)
+
+    new_user = root.subroutes[0]
+    assert new_user.path == "/api/v0/users"
+    new_item = new_user.subroutes[0]
+    assert new_item.path == "/api/v0/users/items"
+    assert new_item.subroutes[0].path == "/api/v0/users/items/{item_id}"
+
+
+async def test_routing_query_with_sequence_type():
+    lc = LocalClient()
+
+    async def get_user(names: tuple[str, ...]):
+        assert isinstance(names, tuple)
+        assert names == ("a", "b", "c")
+
+    ep = await lc.make_endpoint(get_user)
+
+    resp = await lc.request(
+        ep, method="GET", path="test", query_string=b"names=a&names=b&names=c"
+    )
+    assert resp.status_code == 200
+
+
+async def test_routing_query_with_bytes():
+    lc = LocalClient()
+
+    async def get_user(names: bytes): ...
+
+    ep = await lc.make_endpoint(get_user)
+
+    resp = await lc.request(
+        ep, method="GET", path="test", query_string=b"names=a&names=b&names=c"
+    )
+    assert resp.status_code == 200
+
+
+from pydantic import BaseModel
+
+
+class User(BaseModel):
+    name: str
+    age: int
+
+
+async def test_route_with_pydantic_return():
+    async def create_user() -> User:
+        return User(name="1", age=2)
+
+    lc = LocalClient()
+    ep = await lc.make_endpoint(create_user)
+
+    resp = await lc(ep)
+    data = await resp.json()
+    assert data == {"name": "1", "age": 2}
+
+
+async def test_route_with_pydantic_body():
+    async def create_user(user: User) -> User:
+        return user
+
+    lc = LocalClient()
+    ep = await lc.make_endpoint(create_user)
+
+    assert ep.sig.body_param
+
+    resp = await lc(ep, body=User(name="1", age=2))
+    data = await resp.json()
+    assert data == {"name": "1", "age": 2}
