@@ -3,13 +3,14 @@ from typing import Annotated, Union
 import pytest
 from msgspec import Struct
 
-from lihil import Empty, HTTPException, Payload, Resp, Route, Text, status
-from lihil.auth.oauth import OAuth2PasswordFlow
+from lihil import Empty, HTTPException, Payload, Route, Text, status
 from lihil.config import OASConfig
 from lihil.interface import is_set
+from lihil.local_client import LocalClient
 from lihil.oas import get_doc_route, get_openapi_route, get_problem_route
 from lihil.oas.doc_ui import get_problem_ui_html
 from lihil.oas.schema import (
+    json_schema,
     detail_base_to_content,
     generate_oas,
     generate_op_from_ep,
@@ -17,7 +18,7 @@ from lihil.oas.schema import (
     get_path_item_from_route,
     get_resp_schemas,
 )
-from lihil.plugins.testclient import LocalClient
+from lihil.plugins.auth.oauth import OAuth2PasswordFlow
 from lihil.problems import collect_problems
 from lihil.routing import EndpointProps
 
@@ -32,7 +33,11 @@ class Order(Payload, tag=True):
     price: float
 
 
-user_route = Route("/user/{user_id}/order/{order_id}")
+@pytest.fixture
+async def user_route():
+    route = Route("/user/{user_id}/order/{order_id}")
+    await route.setup()
+    return route
 
 
 class OrderNotFound(HTTPException[str]):
@@ -42,7 +47,7 @@ class OrderNotFound(HTTPException[str]):
 oas_config = OASConfig()
 
 
-def test_get_order_schema():
+async def test_get_order_schema(user_route: Route):
     async def get_order(
         user_id: str | int, order_id: str, q: int | str, l: str, u: User
     ) -> Order | User: ...
@@ -50,23 +55,23 @@ def test_get_order_schema():
     user_route.post(errors=OrderNotFound)(get_order)
 
     current_ep = user_route.endpoints["POST"]
-    current_ep.setup()
+    await user_route.setup()
     ep_rt = current_ep.sig.return_params[200]
     ep_rt.type_ == Union[Order, User]
     components = {"schemas": {}}
     ep_oas = generate_op_from_ep(
-        current_ep, components["schemas"], {}, oas_config.problem_path
+        current_ep, components["schemas"], {}, oas_config.PROBLEM_PATH
     )
 
 
-def test_get_hello_return():
+async def test_get_hello_return(user_route: Route):
     @user_route.get
     async def get_hello(
         user_id: str, order_id: str, q: int, l: str, u: User
-    ) -> Resp[Text, status.OK]: ...
+    ) -> Annotated[Text, status.OK]: ...
 
     current_ep = user_route.get_endpoint(get_hello)
-    current_ep.setup()
+    await user_route.setup()
     ep_rt = current_ep.sig.return_params[200]
     assert ep_rt.type_ == bytes
 
@@ -96,14 +101,14 @@ def complex_route():
     return Route("user")
 
 
-def test_complex_route(complex_route: Route):
+async def test_complex_route(complex_route: Route):
 
     class UserNotFoundError(HTTPException[str]):
         "You can't see me"
 
         __status__ = 404
 
-    async def get_user(user_id: str | int) -> Resp[Text, status.OK]:
+    async def get_user(user_id: str | int) -> Annotated[Text, status.OK]:
         if user_id != "5":
             raise UserNotFoundError("You can't see me!")
 
@@ -112,7 +117,7 @@ def test_complex_route(complex_route: Route):
     complex_route.add_endpoint(
         "GET", func=get_user, errors=[UserNotFoundError, UserNotHappyError]
     )
-    complex_route.setup()
+    await complex_route.setup()
 
     oas = generate_oas([complex_route], oas_config, "0.1.0")
     assert oas
@@ -154,12 +159,12 @@ async def test_ep_with_empty_resp():
     route.get(empty_ep)
 
     ep = route.get_endpoint("GET")
-    ep.setup()
+    await route.setup()
     schema = get_resp_schemas(ep, {}, "")
     assert schema["200"].description == "No Content"
 
 
-type MyAlias = Annotated[Annotated[str, "hha"], "aloha"]
+MyAlias = Annotated[Annotated[str, "hha"], "aloha"]
 
 
 async def test_ep_with_annotated_resp():
@@ -171,7 +176,7 @@ async def test_ep_with_annotated_resp():
     route.get(empty_ep)
 
     ep = route.get_endpoint("GET")
-    ep.setup()
+    await route.setup()
     schema = get_resp_schemas(ep, {}, "")
     assert schema
 
@@ -203,31 +208,31 @@ def test_detail_base_to_content():
     assert detail_base_to_content(Random, {}, {})
 
 
-def test_ep_with_status_larger_than_300():
+async def test_ep_with_status_larger_than_300():
     async def create_user() -> (
-        Resp[str, status.NOT_FOUND] | Resp[int, status.INTERNAL_SERVER_ERROR]
+        Annotated[str, status.NOT_FOUND] | Annotated[int, status.INTERNAL_SERVER_ERROR]
     ): ...
 
     route = Route()
     route.post(create_user)
     ep = route.get_endpoint(create_user)
-    ep.setup()
+    await route.setup()
 
     get_resp_schemas(ep, {}, "")
 
 
-def test_ep_without_ret():
+async def test_ep_without_ret():
     async def create_user(): ...
 
     route = Route()
     route.post(create_user)
     ep = route.get_endpoint(create_user)
-    ep.setup()
+    await route.setup()
 
     get_resp_schemas(ep, {}, "")
 
 
-def test_ep_with_auth():
+async def test_ep_with_auth():
 
     async def get_user(token: str): ...
 
@@ -235,27 +240,60 @@ def test_ep_with_auth():
     route.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_user)
 
     ep = route.get_endpoint("GET")
-    ep.setup()
+    await route.setup()
 
     sc = {}
     get_ep_security(ep, sc)
     assert sc["OAuth2PasswordBearer"]
 
 
-def test_ep_with_mutliple_ret():
-    async def f() -> Resp[str, status.OK] | Resp[int | list[int], status.CREATED]: ...
+async def test_ep_with_mutliple_ret():
+    async def f() -> (
+        Annotated[str, status.OK] | Annotated[int | list[int], status.CREATED]
+    ): ...
 
     lc = LocalClient()
 
-    ep = lc.make_endpoint(f)
+    ep = await lc.make_endpoint(f)
 
     get_resp_schemas(ep, {}, "")
 
 
-def test_ep_with_auth_scheme():
-    async def f() -> Resp[str, status.OK] | Resp[int | list[int], status.CREATED]: ...
+async def test_ep_with_auth_scheme():
+    async def f() -> (
+        Annotated[str, status.OK] | Annotated[int | list[int], status.CREATED]
+    ): ...
 
     lc = LocalClient()
 
-    ep = lc.make_endpoint(f)
+    ep = await lc.make_endpoint(f)
     get_resp_schemas(ep, {}, "")
+
+
+from pydantic import BaseModel
+
+
+class PydanticBody(BaseModel):
+    name: str
+    age: str
+
+
+class PydanticResp(BaseModel):
+    email: str
+
+
+async def test_route_with_pydantic_schema():
+
+    async def create_user(user: PydanticBody) -> PydanticResp: ...
+
+    lc = LocalClient()
+    ep = await lc.make_endpoint(create_user)
+
+    result = generate_op_from_ep(ep, {}, {}, "problems")
+    assert result
+
+def test_json_schema_of_msgspec_and_pydantic():
+    from lihil.plugins.auth.supabase import auth_types
+
+    result = json_schema(auth_types.User)
+    assert result
